import os
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse
from requests_oauthlib import OAuth2Session
from genomelink import api_base
from genomelink.errors import raise_oauth_error


class OAuth(object):
    @staticmethod
    def authorize_url(client_id='', scope=[], callback_url=''):
        if not client_id:
            client_id = os.environ['GENOMELINK_CLIENT_ID']
        if not callback_url:
            callback_url = os.environ['GENOMELINK_CALLBACK_URL']

        path = '{}/oauth/authorize'.format(api_base)
        session = OAuth2Session(client_id, scope=scope, redirect_uri=callback_url)
        url, state = session.authorization_url(path)
        return url

    @staticmethod
    def token(client_id='', client_secret='', callback_url='', request_url=''):
        if not client_id:
            client_id = os.environ['GENOMELINK_CLIENT_ID']
        if not client_secret:
            client_secret = os.environ['GENOMELINK_CLIENT_SECRET']
        if not callback_url:
            callback_url = os.environ['GENOMELINK_CALLBACK_URL']

        query = urlparse.parse_qs(urlparse.urlparse(request_url).query)
        error = query.get('error')
        if error:
            error_code = error[0]
            raise_oauth_error(error_code)

        path = '{}/oauth/token'.format(api_base)
        session = OAuth2Session(client_id, redirect_uri=callback_url)
        token = session.fetch_token(path,
                                    client_secret=client_secret,
                                    authorization_response=request_url)
        return token



import os
import unittest
try:
    from unittest.mock import patch, Mock
except ImportError:
    from mock import patch, Mock
try:
    from urllib.parse import urlparse, parse_qs
except ImportError:
    from urlparse import urlparse, parse_qs
import genomelink


class OAuthTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        super(OAuthTestCase, cls).setUpClass()
        os.environ['GENOMELINK_CLIENT_ID'] = 'GENOMELINK_CLIENT_ID'
        os.environ['GENOMELINK_CLIENT_SECRET'] = 'GENOMELINK_CLIENT_SECRET'
        os.environ['GENOMELINK_CALLBACK_URL'] = 'https://localhost:5000/callback'

    @classmethod
    def tearDownClass(cls):
        super(OAuthTestCase, cls).tearDownClass()

    def test_authorize_url(self):
        authorize_url = genomelink.OAuth.authorize_url(scope=['report:eye-color report:beard-thickness report:morning-person'])
        url = urlparse(authorize_url)
        query = parse_qs(url.query)

        self.assertEqual(url.scheme, 'https')
        self.assertEqual(url.netloc, 'genomelink.io')
        self.assertEqual(url.path, '/oauth/authorize')
        self.assertEqual(query['response_type'], ['code'])
        self.assertEqual(query['client_id'], [os.environ['GENOMELINK_CLIENT_ID']])
        self.assertEqual(query['redirect_uri'], [os.environ['GENOMELINK_CALLBACK_URL']])
        self.assertEqual(query['scope'], ['report:eye-color report:beard-thickness report:morning-person'])
        self.assertTrue(query.get('state'))

    @patch('requests.Session.post')
    def test_token(self, mock_post):
        mock_post.return_value = Mock(text='''
        {
          "access_token":"MTQ0NjJkZmQ5OTM2NDE1ZTZjNGZmZjI3",
          "token_type":"bearer",
          "expires_in":3600,
          "refresh_token":"IwOGYzYTlmM2YxOTQ5MGE3YmNmMDFkNTVk",
          "scope":"report:eye-color report:beard-thickness report:morning-person",
          "state":"12345678"
        }
        ''')

        token = genomelink.OAuth.token(request_url='https://localhost:5000/callback?code=xxx')
        self.assertEqual(token['access_token'], 'MTQ0NjJkZmQ5OTM2NDE1ZTZjNGZmZjI3')



import os
import requests
from requests_oauthlib import OAuth2Session
from genomelink import api_base
from genomelink.errors import GenomeLinkError


class Report(object):
    @staticmethod
    def fetch(name, population, token='', client_secret=''):
        if type(token) == str:
            token = {'token_type': 'Bearer', 'access_token': token}

        if token['access_token'].startswith('T_'):
            if not client_secret:
                client_secret = os.environ.get('GENOMELINK_CLIENT_SECRET')
                if not client_secret:
                    raise GenomeLinkError('GENOMELINK_CLIENT_SECRET is not provided.')

            path = '{}/v1/enterprise/reports/{name}/?population={population}'.format(api_base, name=name, population=population)
            response = requests.post(path,
                                     headers={'Authorization': 'Bearer {}'.format(client_secret)},
                                     data={'token': token['access_token']})
            if response.status_code != requests.codes.ok:
                raise GenomeLinkError('Invalid request.')

            record = [Report(report) for report in response.json()['reports']]

        else:
            path = '{}/v1/reports/{name}/?population={population}'.format(api_base, name=name, population=population)
            session = OAuth2Session(token=token)
            response = session.get(path)
            record = Report(response.json())

        return record

    def __init__(self, data):
        self._data = data

    @property
    def phenotype(self):
        return self._data['phenotype']

    @property
    def population(self):
        return self._data['population']

    @property
    def scores(self):
        return self._data['scores']

    @property
    def summary(self):
        return self._data['summary']
